Index: vlc_internal.c
===================================================================
--- vlc_internal.c	(revision 20336)
+++ vlc_internal.c	(working copy)
@@ -122,7 +122,7 @@
 vlcObject_dealloc( PyObject *self )
 {
     vlcObject_release( self, NULL );
-    PyMem_DEL( self );
+    PyObject_DEL( self );
 }
 
 static PyObject *
Index: vlc_mediacontrol.c
===================================================================
--- vlc_mediacontrol.c	(revision 20336)
+++ vlc_mediacontrol.c	(working copy)
@@ -42,7 +42,6 @@
 
     if( PyArg_ParseTuple( args, "O", &py_param ) )
     {
-        Py_INCREF( py_param );
         if( PyObject_TypeCheck( py_param, &vlcInstance_Type ) == 1 )
         {
             p_instance = ((vlcInstance*)py_param)->p_instance;
@@ -51,6 +50,7 @@
         {
             Py_ssize_t i_index;
 
+            Py_INCREF( py_param );
             if( ! PySequence_Check( py_param ) )
             {
                 PyErr_SetString( PyExc_TypeError, "Parameter must be a vlc.Instance or a sequence of strings." );
@@ -74,8 +74,8 @@
                                                                        i_index ) ) ) );
             }
             ppsz_args[i_size] = NULL;
+            Py_DECREF( py_param );
         }
-        Py_DECREF( py_param );
     }
     else
     {
@@ -89,10 +89,14 @@
     if( p_instance )
     {
         self->mc = mediacontrol_new_from_instance( p_instance, exception );
+        Py_INCREF( py_param );
+        self->vlc_instance = py_param;
     }
     else
     {
         self->mc = mediacontrol_new( i_size, ppsz_args, exception );
+        self->vlc_instance = PyObject_New( vlcInstance, &vlcInstance_Type );
+        self->vlc_instance->p_instance = mediacontrol_get_libvlc_instance( SELF->mc );
     }
     MC_EXCEPT;
     Py_END_ALLOW_THREADS
@@ -104,7 +108,8 @@
 static void
 MediaControl_dealloc( PyObject *self )
 {
-    PyMem_DEL( self );
+    Py_DECREF( SELF->vlc_instance );
+    PyObject_DEL( self );
 }
 
 static PyObject *
@@ -112,10 +117,8 @@
 {
     vlcInstance *p_ret;
 
-    p_ret = PyObject_New( vlcInstance, &vlcInstance_Type );
-    if( !p_ret )
-        return NULL;
-    p_ret->p_instance = mediacontrol_get_libvlc_instance( SELF->mc );
+    p_ret = SELF->vlc_instance;
+    Py_INCREF( p_ret );
     return ( PyObject * )p_ret;
 }
 
Index: vlcglue.h
===================================================================
--- vlcglue.h	(revision 20336)
+++ vlcglue.h	(working copy)
@@ -81,12 +81,24 @@
 PyObject *vlcInstance_Exception;
 
 /**********************************************************************
+ * vlc.Instance Object
+ **********************************************************************/
+typedef struct
+{
+    PyObject_HEAD
+    libvlc_instance_t* p_instance;
+} vlcInstance;
+
+#define LIBVLC_INSTANCE ((vlcInstance*)self)
+
+/**********************************************************************
  * MediaControl Object
  **********************************************************************/
 typedef struct
 {
     PyObject_HEAD
     mediacontrol_Instance* mc;
+    vlcInstance *vlc_instance;
 } MediaControl;
 
 /**********************************************************************
@@ -101,17 +113,6 @@
 } PyPosition;
 
 /**********************************************************************
- * vlc.Instance Object
- **********************************************************************/
-typedef struct
-{
-    PyObject_HEAD
-    libvlc_instance_t* p_instance;
-} vlcInstance;
-
-#define LIBVLC_INSTANCE ((vlcInstance*)self)
-
-/**********************************************************************
  * vlc.Input Object
  **********************************************************************/
 typedef struct
Index: vlc_instance.c
===================================================================
--- vlc_instance.c	(revision 20336)
+++ vlc_instance.c	(working copy)
@@ -82,9 +83,6 @@
     char** ppsz_args = NULL;
     int i_size = 0;
 
-    self = PyObject_New( vlcInstance, &vlcInstance_Type );
-
-
     if( PyArg_ParseTuple( args, "O", &py_list ) )
     {
         i_size = pyoptions_to_args( py_list, &ppsz_args );
@@ -98,6 +96,8 @@
         PyErr_Clear( );
     }
 
+    self = PyObject_New( vlcInstance, &vlcInstance_Type );
+
     Py_BEGIN_ALLOW_THREADS
     LIBVLC_TRY
     LIBVLC_INSTANCE->p_instance = libvlc_new( i_size, ppsz_args, &ex );
@@ -114,7 +114,7 @@
 {
     libvlc_exception_t ex;
     libvlc_destroy( LIBVLC_INSTANCE->p_instance, &ex );
-    PyMem_DEL( self );
+    PyObject_DEL( self );
 }
 
 static PyObject *
